.bk.card {
  border: 1px solid rgba(0,0,0,.125);
  border-radius: 0.25rem;
}
.bk.accordion {
  border: 1px solid rgba(0,0,0,.125);
}
.bk.card-header {
  align-items: center;
  background-color: rgba(0, 0, 0, 0.03);
  border-radius: 0.25rem;
  display: flex;
  justify-content: space-between;
  padding: 0 1.25rem 0 0;
  width: 100%;
}
.bk.accordion-header {
  align-items: center;
  background-color: rgba(0, 0, 0, 0.03);
  border-radius: 0;
  display: flex;
  justify-content: space-between;
  padding: 0 1.25rem 0 0;
  width: 100%;
}
p.bk.card-button {
  background-color: transparent;
  font-size: 1.25rem;
  font-weight: 700;
  margin: 0;
  margin-left: -15px;
}
.bk.card-header-row {
  position: relative !important;
}
.bk.card-title {
  align-items: center;
  display: flex !important;
  font-size: 1.4em;
  font-weight: bold;
  padding: 0.25em;
  position: relative !important;
}
