{% extends 'base.html' %}
{% load static %}


{% block content %}

    <div class="jumbotron jumbotron-fluid bg-danger text-center">
        <h1>Landing Page</h1>
        <p class="lead">Predictive Analytics</p>
    </div>


{%  endblock content %}
